
Imports EpsonPLQDriver

' CAPIControl
Public Class APIControl

    Protected m_objMainForm As MainForm = Nothing
    Protected WithEvents m_objMfDevice As MFDevice = Nothing
    Protected m_objMfBase As MFBase = Nothing
    Protected m_objMfMicr As MFMicr = Nothing
    Protected m_objMfMsrw As MFMsrw = Nothing
    Protected m_ErrFlag As Boolean = False

    Protected Const PAPER_INSERT_TIMEOUT As Integer = 10
    Protected Const PASSBOOK_INSERT_TIMEOUT As Integer = 10000
    Protected Const PAPER_EJECT_TIMEOUT As Integer = 5000

    Public Sub New()
        m_objMfDevice = New MFDevice()
        m_objMfBase = New MFBase
        m_objMfMicr = New MFMicr
        m_objMfMsrw = New MFMsrw
        m_ErrFlag = False
    End Sub

    Public Sub MainFormItem(ByVal Item As MainForm)
        m_objMainForm = Item
    End Sub

    Public Function InitInstance() As Boolean
        Dim eErrorCode As ErrorCode = ErrorCode.SUCCESS

        ' Connect to device
        ' If connection failed, this program displays a message and tries to reconnect.
        Do
            eErrorCode = m_objMfDevice.OpenMonPrinter(OpenType.TYPE_PRINTER, "PLQ-22U")
            If eErrorCode <> ErrorCode.SUCCESS Then
                If MessageBox.Show("Unable to connect to printer" & vbCrLf & "Ensure Driver installed correctly" & vbCrLf & "and Printer is powered on", _
                        eErrorCode.ToString(), MessageBoxButtons.OKCancel, MessageBoxIcon.Error) = DialogResult.Cancel Then
                    Return False
                End If
                System.Threading.Thread.Sleep(1000)
            End If
        Loop Until eErrorCode = ErrorCode.SUCCESS

        ' Register callback function for device status notification
        CheckResponse(m_objMfDevice.SetMultiStatusBack(), "SetMultiStatusBack()")
        If m_ErrFlag Then
            Return False
        End If

        Return True
    End Function

    Public Sub Close()
        If Not (m_objMfDevice Is Nothing) Then
            If m_objMfDevice.IsValid Then
                ' Cancel callback function for device status notification
                CheckResponse(m_objMfDevice.CancelStatusBack(), "CancelStatusBack()")
                ' Disconnect from device
                CheckResponse(m_objMfDevice.CloseMonPrinter(), "CloseMonPrinter()")
            End If
        End If
    End Sub

    ' Read MICR
    Public Function ReadMICR(ByVal eMfMicrType As MfMicrType, ByVal eMfMicrFont As MfMicrFont, ByVal bSave As Boolean) As String
        Dim strMICR As String = ""
        Dim eErrorCode As ErrorCode = ErrorCode.SUCCESS

        ' Lock the port
        CheckResponse(m_objMfDevice.LockPort(0), "LockPort(0)")
        If m_ErrFlag Then
            Return strMICR
        End If

        ' Set MICR settings in driver
        ConfigureMICR(eMfMicrType, eMfMicrFont)
        If m_ErrFlag Then
            ' Unlock the port
            CheckResponse(m_objMfDevice.UnlockPort(), "UnlockPort()")
            Return strMICR
        End If

        ' Process MICR
        eErrorCode = m_objMfDevice.SCNMICRFunction(FunctionType.MF_EXEC)
        strMICR = m_objMfMicr.MicrStr
        If eErrorCode <> ErrorCode.SUCCESS Then
            strMICR &= " ( "
            strMICR &= eErrorCode.ToString()
            strMICR &= " ) "
        Else
            If m_objMfMicr.Ret <> ErrorCode.SUCCESS Then
                strMICR &= " ( "
                strMICR &= m_objMfMicr.Ret.ToString()
                strMICR &= " ) "
            End If
        End If

        ' Unlock the port
        CheckResponse(m_objMfDevice.UnlockPort(), "UnlockPort()")

        If bSave Then
            SaveMICR(eErrorCode)
        End If

        Return strMICR
    End Function

    ' Read magnetic stripe
    Public Function ReadStripe(ByVal eMsrwFormat As MsrwFormat, ByVal eMsrwIBMDataTyoe As MsrwIBMDataType, ByVal bSave As Boolean) As String
        Dim strMSR As String = ""
        Dim strSave As String = ""
        Dim eErrorCode As ErrorCode = ErrorCode.SUCCESS
        Dim iReadLength As UInteger = 0
        Dim iDetail As UInteger = 0
        Dim bRead As Byte() = Nothing

        ' Lock the port
        CheckResponse(m_objMfDevice.LockPort(0), "LockPort(0)")
        If m_ErrFlag Then
            Return strMSR
        End If

        ' Set magnetic stripe read settings in driver
        ConfigureMSRW(eMsrwFormat, eMsrwIBMDataTyoe)

        ' Set MF_MSRW class in driver
        eErrorCode = m_objMfDevice.MSRWSetting(m_objMfMsrw)
        If eErrorCode = ErrorCode.SUCCESS Then
            ' Process magnetic stripe reading
            eErrorCode = m_objMfDevice.MSRWReadStripeData(PASSBOOK_INSERT_TIMEOUT)
            If eErrorCode = ErrorCode.SUCCESS Then

                ' Retrieve data size for magnetic stripe
                eErrorCode = m_objMfDevice.MSRWGetStripeData(1, Nothing, iReadLength, iDetail)

                If iReadLength <> 0 Then
                    ReDim bRead(CInt(iReadLength - 1))
                End If

                ' Retrieve data for magnetic stripe
                eErrorCode = m_objMfDevice.MSRWGetStripeData(1, bRead, iReadLength, iDetail)
                If iReadLength <> 0 Then
                    strMSR = System.Text.Encoding.ASCII.GetString(bRead, 0, bRead.Length - 1)
                    strSave = System.Text.Encoding.ASCII.GetString(bRead, 0, bRead.Length - 1)
                Else
                    If eErrorCode = ErrorCode.SUCCESS Then
                        strMSR = " < No Data > "
                    End If
                End If
            End If

            ' Eject passbook
            CheckResponse(m_objMfDevice.EjectPaper(MfEjectType.MF_EJECT_FRONT, PAPER_EJECT_TIMEOUT), _
                                        "EjectPaper(MfEjectType.MF_EJECT_FRONT, PAPER_EJECT_TIMEOUT)")
        End If

        ' Unlock the port
        CheckResponse(m_objMfDevice.UnlockPort(), "UnlockPort()")

        If eErrorCode <> ErrorCode.SUCCESS Then
            strMSR &= " ( "
            strMSR &= eErrorCode.ToString()
            strMSR &= " ) "
        End If

        If bSave Then
            SaveMSR(eErrorCode, iDetail, strSave)
        End If

        Return strMSR
    End Function

    ' Write magnetic stripe
    Public Sub WriteStripe(ByVal eMsrwFormat As MsrwFormat, ByVal eMsrwIBMDataTyoe As MsrwIBMDataType, ByVal strWrite As String)
        ' Lock the port
        CheckResponse(m_objMfDevice.LockPort(0), "LockPort(0)")
        If m_ErrFlag Then
            Exit Sub
        End If

        ' Set magnetic stripe read settings in driver
        ConfigureMSRW(eMsrwFormat, eMsrwIBMDataTyoe)

        ' Set MF_MSRW class in driver
        CheckResponse(m_objMfDevice.MSRWSetting(m_objMfMsrw), "MSRWSetting(m_objMfMsrw)")
        If m_ErrFlag Then
            ' Unlock the port
            CheckResponse(m_objMfDevice.UnlockPort(), "UnlockPort()")
            Exit Sub
        End If

        ' Process magnetic stripe writing
        CheckResponse(m_objMfDevice.MSRWWriteStripeData(PASSBOOK_INSERT_TIMEOUT, System.Text.Encoding.ASCII.GetBytes(strWrite)), _
                                    "MSRWWriteStripeData(PASSBOOK_INSERT_TIMEOUT, System.Text.Encoding.ASCII.GetBytes(strWrite))")

        ' Eject passbook
        CheckResponse(m_objMfDevice.EjectPaper(MfEjectType.MF_EJECT_FRONT, PAPER_EJECT_TIMEOUT), _
                                    "EjectPaper(MfEjectType.MF_EJECT_FRONT, PAPER_EJECT_TIMEOUT)")

        ' Unlock the port
        CheckResponse(m_objMfDevice.UnlockPort(), "UnlockPort()")
    End Sub

    ' Erase magnetic stripe
    Public Sub ClearStripe(ByVal eMsrwFormat As MsrwFormat, ByVal eMsrwIBMDataTyoe As MsrwIBMDataType)
        ' Lock the port
        CheckResponse(m_objMfDevice.LockPort(0), "LockPort(0)")
        If m_ErrFlag Then
            Exit Sub
        End If

        ' Set magnetic stripe read settings in driver
        ConfigureMSRW(eMsrwFormat, eMsrwIBMDataTyoe)

        ' Set MF_MSRW class in driver
        CheckResponse(m_objMfDevice.MSRWSetting(m_objMfMsrw), "MSRWSetting(m_objMfMsrw)")
        If m_ErrFlag Then
            ' Unlock the port
            CheckResponse(m_objMfDevice.UnlockPort(), "UnlockPort()")
            Exit Sub
        End If

        ' Process magnetic stripe erasing
        CheckResponse(m_objMfDevice.MSRWClearStripeData(PASSBOOK_INSERT_TIMEOUT), _
                                    "MSRWClearStripeData(PASSBOOK_INSERT_TIMEOUT)")

        ' Eject passbook
        CheckResponse(m_objMfDevice.EjectPaper(MfEjectType.MF_EJECT_FRONT, PAPER_EJECT_TIMEOUT), _
                                    "EjectPaper(MfEjectType.MF_EJECT_FRONT, PAPER_EJECT_TIMEOUT)")

        ' Unlock the port
        CheckResponse(m_objMfDevice.UnlockPort(), "UnlockPort()")
    End Sub

    ' Setting MICR settings
    Private Sub ConfigureMICR(ByVal eMfMicrType As MfMicrType, ByVal eMfMicrFont As MfMicrFont)
        ' MF_BASE01
        ' Retrieve default values for MF_BASE01 class
        CheckResponse(m_objMfDevice.SCNMICRFunction(m_objMfBase, FunctionType.MF_GET_BASE_DEFAULT), _
                                                    "SCNMICRFunction(m_objMfBase, FunctionType.MF_GET_BASE_DEFAULT)")
        If m_ErrFlag Then
            Exit Sub
        End If

        m_objMfBase.Timeout = PAPER_INSERT_TIMEOUT

        ' Set class MF_BASE01 in driver
        CheckResponse(m_objMfDevice.SCNMICRFunction(m_objMfBase, FunctionType.MF_SET_BASE_PARAM), _
                                                    "SCNMICRFunction(m_objMfBase, FunctionType.MF_SET_BASE_PARAM)")
        If m_ErrFlag Then
            Exit Sub
        End If

        ' MF_MICR
        ' Retrieve default values for MF_MICR class
        CheckResponse(m_objMfDevice.SCNMICRFunction(m_objMfMicr, FunctionType.MF_GET_MICR_DEFAULT), _
                                                    "SCNMICRFunction(m_objMfMicr, FunctionType.MF_GET_MICR_DEFAULT)")
        If m_ErrFlag Then
            Exit Sub
        End If

        m_objMfMicr.Font = eMfMicrFont
        m_objMfMicr.MicrOcrSelect = eMfMicrType

        ' Set MF_MICR class in driver
        CheckResponse(m_objMfDevice.SCNMICRFunction(m_objMfMicr, FunctionType.MF_SET_MICR_PARAM), _
                                                    "SCNMICRFunction(m_objMfMicr, FunctionType.MF_SET_MICR_PARAM)")
        If m_ErrFlag Then
            Exit Sub
        End If
    End Sub

    ' Settings for magnetic stripe reading, writing, and erasing
    Private Sub ConfigureMSRW(ByVal eMsrwFormat As MsrwFormat, ByVal eMsrwIBMDataTyoe As MsrwIBMDataType)
        ' MF_MSRW
        m_objMfMsrw.RecordFormat = eMsrwFormat
        m_objMfMsrw.IBMDataType = eMsrwIBMDataTyoe
        m_objMfMsrw.ReadRetry = 0
        m_objMfMsrw.WriteAndClearRetry = 0
        m_objMfMsrw.BlockCount = 1
        m_objMfMsrw.VerticalAdjust = 0
    End Sub

    ' Saving MICR data to file
    Private Sub SaveMICR(ByVal errCode As ErrorCode)
        Dim strOutput As String = ""

        strOutput &= "# MICR Data Output" & vbCrLf & vbCrLf
        strOutput &= "# Magnetic Character Results" & vbCrLf
        strOutput &= "Result = " & m_objMfMicr.Ret.ToString() & vbCrLf
        strOutput &= "Status = " & m_objMfMicr.Status.ToString() & vbCrLf
        strOutput &= "Detail = " & m_objMfMicr.Detail.ToString() & vbCrLf & vbCrLf
        strOutput &= "MicrResult = " & errCode.ToString() & vbCrLf
        strOutput &= "MicrString = " & m_objMfMicr.MicrStr & vbCrLf

        Dim sw As System.IO.StreamWriter = New System.IO.StreamWriter("Micr.txt")
        sw.Write(strOutput)
        sw.Close()
    End Sub

    ' Saving magnetic stripe data to file
    Private Sub SaveMSR(ByVal errCode As ErrorCode, ByVal iDetail As UInteger, ByVal strStripeData As String)
        Dim strOutput As String = ""

        strOutput &= "# MSR Data Output" & vbCrLf & vbCrLf
        strOutput &= "# Magnetic Stripe Results" & vbCrLf
        strOutput &= "RecordFormat = " & GetRecordFormatString(m_objMfMsrw.RecordFormat) & vbCrLf
        strOutput &= "IBMDataType = " & GetIBMDataTypeString(m_objMfMsrw.IBMDataType) & vbCrLf
        strOutput &= "Detail = " & iDetail.ToString() & vbCrLf & vbCrLf
        strOutput &= "MSRResult = " & errCode.ToString() & vbCrLf
        strOutput &= "MSRData = " & strStripeData & vbCrLf

        Dim sw As System.IO.StreamWriter = New System.IO.StreamWriter("MSR.txt")
        sw.Write(strOutput)
        sw.Close()
    End Sub

    ' Get device status
    Public Function GetStatus() As EscMcASB()
        Dim nCount As Integer = 0
        Dim eEscMcASB As EscMcASB() = Nothing

        eEscMcASB = m_objMfDevice.MultiStatus
        CheckResponse(m_objMfDevice.LastError, "MultiStatus")

        Return eEscMcASB
    End Function

    ' Callback function called when device status notification occurs
    Private Sub MultiStatus(ByVal eEscMcASBType As EscMcASBType, ByVal eEscMcASB As EscMcASB, ByVal str As String) Handles m_objMfDevice.MultiStatusCallback
        m_objMainForm.SetDeviceStatusCallback(eEscMcASBType, eEscMcASB)
    End Sub

    ' This macro checks the return value of PLQ-22 API.
    ' If an error is returned, an error message is displayed and the error flag is set to true.
    Private Sub CheckResponse(ByVal errCode As ErrorCode, ByVal strmessage As String)
        m_ErrFlag = False
        If errCode <> ErrorCode.SUCCESS Then
            MessageBox.Show(strmessage, errCode.ToString(), MessageBoxButtons.OK, MessageBoxIcon.Error)
            m_ErrFlag = True
        End If
    End Sub

    Private Function GetRecordFormatString(ByVal eMsrwFormat As MsrwFormat) As String
        Dim strFormat As String = ""

        Select Case eMsrwFormat
            Case MsrwFormat.EPS_BI_MSRW_FORMAT_IBM3604
                strFormat = "IBM3604"
            Case MsrwFormat.EPS_BI_MSRW_FORMAT_DINISO
                strFormat = "DINISO"
            Case MsrwFormat.EPS_BI_MSRW_FORMAT_ANSI
                strFormat = "ANSI"
            Case MsrwFormat.EPS_BI_MSRW_FORMAT_IBM4746
                strFormat = "IBM4746"
            Case MsrwFormat.EPS_BI_MSRW_FORMAT_ISO7811
                strFormat = "ISO7811"
            Case MsrwFormat.EPS_BI_MSRW_FORMAT_HT2751CIZ
                strFormat = "HT2751CIZ"
            Case MsrwFormat.EPS_BI_MSRW_FORMAT_ISO8484
                strFormat = "ISO8484"
            Case MsrwFormat.EPS_BI_MSRW_FORMAT_IBM3604
                strFormat = "IBM3604"
            Case Else
                strFormat = CInt(eMsrwFormat).ToString()
        End Select

        Return strFormat
    End Function

    Private Function GetIBMDataTypeString(ByVal eMsrwIBMDataType As MsrwIBMDataType) As String
        Dim strDataType As String = ""

        Select Case eMsrwIBMDataType
            Case MsrwIBMDataType.EPS_BI_MSRW_EOM_NONE
                strDataType = "NONE"
            Case MsrwIBMDataType.EPS_BI_MSRW_EOM_3C
                strDataType = "3C"
            Case MsrwIBMDataType.EPS_BI_MSRW_EOM_3F
                strDataType = "3F"
            Case Else
                strDataType = CInt(eMsrwIBMDataType).ToString()
        End Select

        Return strDataType
    End Function

End Class
